<?php

if(!class_exists('pwaforwp_ptafp')){
	require_once PWAFORWP_PTAFP_PLUGIN_DIR."/inc/common_authentication.php";
	class pwaforwp_ptafp{
		public $settings = array();
		public $ptap_authentication;
		public function __construct(){
			add_action("admin_init", array( $this, 'init' ) );
			add_action( 'plugin_action_links_' . plugin_basename( PWAFORWP_PTAFP_DIR_NAME_FILE ), array($this, 'plugin_action_links') );
			if(function_exists('pwaforwp_defaultSettings')){
				$this->settings = pwaforwp_defaultSettings();
			}else{ $this->settings = array(); }
			
			$this->ptap_authentication =  new ptap_common_authentication();
			add_action("admin_enqueue_scripts", array($this , 'script_enqueue'));

			//on save 
			add_action( "update_option_pwaforwp_settings", array($this, 'on_options_update') ,10,3);

			add_action("wp_ajax_pwa_to_apk_plugin_caller", array($this, "pwa_to_apk_plugin_gen"));
			add_action("wp_ajax_pwa_to_apk_plugin_project_caller", array($this, "pwa_to_apk_plugin_project"));
			add_action("wp_ajax_pwa_to_apk_assets_deeeplink", array($this, "pwa_to_apk_assets_deeeplink"));
			add_action("wp_ajax_pwaforwp_apk_keystore_upload", array($this, "pwaforwp_apk_keystore_upload"));
			add_action("wp_ajax_pwa_to_apk_assets_removeold", array($this, "pwa_to_apk_assets_removeold"));
		}

		public function on_options_update($old_value, $value, $option){
			$pwa_pwatapk = get_option("pwa_pwatapk", true);
			if(!is_array($pwa_pwatapk)){ $pwa_pwatapk = array(); }
			if(isset($pwa_pwatapk["user_authentication"]) && $pwa_pwatapk["user_authentication"]==1){
				return true;
			}
			if( (isset($value['ptafp_addon_license_key']) 
					&& isset($value['ptafp_addon_license_key_status']) 
					&& $value['ptafp_addon_license_key_status'] == 'active'
				) && isset($value['pwa_to_apk_plugin'])
			){
				/*$data = array(
						'license_key' => $value['ptafp_addon_license_key'],
						'license_status'=> $value['ptafp_addon_license_key_status'],
						'referrer'=> get_home_url()
				);
				$response = $this->ptap_authentication->sender_request("registerSiteUser", $data);
				if($response){*/
					$pwa_pwatapk["user_authentication"] = 1;
					update_option("pwa_pwatapk", $pwa_pwatapk);
				//}
			}
		}

		function pwa_to_apk_plugin_gen(){
			$nonce = $_POST['ref'];
			if(!wp_verify_nonce($nonce, 'pwa_pwatapk_nonce_gen')){
				echo json_encode(array("status"=>400, "message"=>"Request not verified"));die;
			}
			$pwa_pwatapk = get_option("pwa_pwatapk", true);
			if( (isset($pwa_pwatapk["user_authentication"]) && $pwa_pwatapk["user_authentication"]!=1)){
				echo json_encode(array("status"=>400, "message"=>"Plugin not activated"));die;
			}
			
			//PWA settings
			$defaults = pwaforwp_defaultSettings();

			$signingMode = 'new'; $keyPassword = $keystorePassword = ''; $keystorefile = null;
			$appVersionCode = 1;
			$version = "1.0.0";
			if(isset($defaults['apk_type']) && $defaults['apk_type']=='mine'){ 
				$signingMode = 'mine';  
				$keyPassword = isset($defaults['apk_key_password'])? $defaults['apk_key_password']: '';
				$keystorePassword = isset($defaults['apk_key_store_password'])? $defaults['apk_key_store_password']: '';
				
				if(!file_exists(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore")){
					echo json_encode(array("status"=>400, "message"=>"Keystore File not found"));die;
				}else{
					$keystorefile = 'data:application/octet-stream;base64,'.base64_encode(file_get_contents(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore"));
				}

				if( empty($keyPassword) ){
					echo json_encode(array("status"=>400, "message"=>"Key password is empty"));die;
				}
				if( empty($keystorePassword) ){
					echo json_encode(array("status"=>400, "message"=>"Key store password is empty"));die;
				}
				if(isset($defaults['apk_app_version']) && !empty($defaults['apk_app_version'])){
					$version = $defaults['apk_app_version'];
				}else{
					echo json_encode(array("status"=>400, "message"=>"Last APK version is empty"));die;
				}
				if( isset($defaults['apk_app_code']) && !empty($defaults['apk_app_code']) ) {
					$appVersionCode = $defaults['apk_app_code']+1;
				}else{
					echo json_encode(array("status"=>400, "message"=>"Last APK code is empty, ex: 1"));die;
				}
				if(version_compare ( $version , "1.0.0" , '>=' ) ){
					//$version = $pwa_pwatapk['last_version'];
					$verArr = explode(".", $version);
					if($verArr[count($verArr)-1]<=9){
						$verArr[count($verArr)-1] = $verArr[count($verArr)-1]+1;
					}elseif($verArr[count($verArr)-2]<=9){
						$verArr[count($verArr)-1] = 0;
						$verArr[count($verArr)-2] = $verArr[count($verArr)-2]+1;
					}elseif($verArr[count($verArr)-3]<=9){
						$verArr[count($verArr)-1] = 0;
						$verArr[count($verArr)-2] = 0;
						$verArr[count($verArr)-3] = $verArr[count($verArr)-3]+1;
					}
					$version = implode(".", $verArr);
				}
			}
			
			
			
			$host = pwaforwp_home_url();
			/*$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
			 if(strpos($host, 'www')!==false){
				$host = str_replace("www.", "", $host);
			} */
			$packageId = $defaults['pta_package_id']; 
			
			$data = array(
			    "packageId"=> $packageId,
			    "host"=> $host,
			    "name"=> esc_attr($defaults['app_blog_name']),
			    "launcherName"=> esc_attr($defaults['app_blog_short_name']),
			    "themeColor"=> sanitize_hex_color($defaults['theme_color']),
			    "navigationColor"=> sanitize_hex_color($defaults['background_color']),
			    "backgroundColor"=> sanitize_hex_color($defaults['background_color']),
			    "startUrl"=> "/",
				"webManifestUrl"=>esc_url( pwaforwp_manifest_json_url() ),
			    "iconUrl"=> esc_url(pwaforwp_https($defaults['icon'])),
			    "maskableIconUrl"=> esc_url(pwaforwp_https($defaults['splash_icon'])),
			    "appVersion"=> $version,
			    "appVersionCode"=> $appVersionCode,
			    "useBrowserOnChromeOS"=> true,
			    "splashScreenFadeOutDuration"=> 300,
			    "enableNotifications"=> true,
				"display"=> esc_attr($defaults['display']),
			    "shortcuts"=> array(array(
								"name"=> esc_attr($defaults['app_blog_name']),
								"short_name"=> esc_attr($defaults['app_blog_short_name']),
								"url"=> "/?shortcut",
								"icons"=> array( /*array(
										"sizes"=> "128x128",
										"src"=> "/favicon.png",
									) */)
							)),
				"signingMode"=> $signingMode,
			    "signing"=> array(
					"file"=> $keystorefile,
					"alias"=> "my-key-alias",
			        "fullName"=> "PWA for wp",
			        "organization"=> "MAGAZINE3",
			        "organizationalUnit"=> "Development",
			        "countryCode"=> "IN",
					"keyPassword"=> $keyPassword,
					"storePassword"=> $keystorePassword
			    ),
				"fallbackType"=> "customtabs",
			);

			$headers_data = array('request_url'=>esc_url(get_home_url()), 'request_license'=>sanitize_text_field($this->settings['ptafp_addon_license_key']));
			set_time_limit(500);
			$response = $this->ptap_authentication->sender_request("generateApkZip", $data, $headers_data, true);
			if($response){
				if( $response['status'] == 200){
					$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
					 if(strpos($host, 'www')!==false){
						$host = str_replace("www.", "", $host);
					} 
					
					$upload_path = wp_upload_dir();
					$file_path = $upload_path['basedir']."/pwa_to_app/";
					wp_mkdir_p($file_path);
					$file_name = $host."-".$version."-".$this->generate_random_string().'.zip';
					$file_name_path = $file_path.'/'.$file_name;
					$fp = file_put_contents($file_name_path, $response['body']);


					$pwa_pwatapk["user_authentication"] = 1;
					$pwa_pwatapk["last_version"] = $version;
					$pwa_pwatapk["old_files"][] = array('name'=>$file_name_path, 'fname'=>$file_name, 'version'=>$version, 'time'=> time());
					update_option("pwa_pwatapk", $pwa_pwatapk);
					$this->pwa_to_apk_assets_deeeplink($version);
					$fileUrl = $upload_path['baseurl']."/pwa_to_app/".$file_name;
					echo json_encode(array("status"=>200, "message"=> "apk downloaded <a href='".$fileUrl."'>Download</a>"));
				}else{
					echo json_encode(array("status"=>300, "message"=> "APK not created try later. <br/>ErrorDetails: ".$response['body']));
				}
			}else{
				echo json_encode(array("status"=>400, "message"=> "Some error occurred in response, try again later."));
			}
			die;
		}

		function pwa_to_apk_plugin_project(){
			$nonce = $_POST['ref'];
			if(!wp_verify_nonce($nonce, 'pwa_pwatapk_nonce_gen')){
				echo json_encode(array("status"=>400, "message"=>"Request not verified"));die;
			}
			$pwa_pwatapk = get_option("pwa_pwatapk", true);
			if( (isset($pwa_pwatapk["user_authentication"]) && $pwa_pwatapk["user_authentication"]!=1)){
				echo json_encode(array("status"=>400, "message"=>"Plugin not activated"));die;
			}
			
			//PWA settings
			$defaults = pwaforwp_defaultSettings();

			$signingMode = 'new'; $keyPassword = $keystorePassword = ''; $keystorefile = null;
			$appVersionCode = 1;
			$version = "1.0.0";
			if(isset($defaults['apk_type']) && $defaults['apk_type']=='mine'){ 
				$signingMode = 'mine';  
				$keyPassword = isset($defaults['apk_key_password'])? $defaults['apk_key_password']: '';
				$keystorePassword = isset($defaults['apk_key_store_password'])? $defaults['apk_key_store_password']: '';
				
				if(!file_exists(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore")){
					echo json_encode(array("status"=>400, "message"=>"Keystore File not found"));die;
				}else{
					$keystorefile = 'data:application/octet-stream;base64,'.base64_encode(file_get_contents(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore"));
				}

				if( empty($keyPassword) ){
					echo json_encode(array("status"=>400, "message"=>"Key password is empty"));die;
				}
				if( empty($keystorePassword) ){
					echo json_encode(array("status"=>400, "message"=>"Key store password is empty"));die;
				}
				if(isset($defaults['apk_app_version']) && !empty($defaults['apk_app_version'])){
					$version = $defaults['apk_app_version'];
				}else{
					echo json_encode(array("status"=>400, "message"=>"Last APK version is empty"));die;
				}
				if( isset($defaults['apk_app_code']) && !empty($defaults['apk_app_code']) ) {
					$appVersionCode = $defaults['apk_app_code']+1;
				}else{
					echo json_encode(array("status"=>400, "message"=>"Last APK code is empty, ex: 1"));die;
				}
				if(version_compare ( $version , "1.0.0" , '>=' ) ){
					//$version = $pwa_pwatapk['last_version'];
					$verArr = explode(".", $version);
					if($verArr[count($verArr)-1]<=9){
						$verArr[count($verArr)-1] = $verArr[count($verArr)-1]+1;
					}elseif($verArr[count($verArr)-2]<=9){
						$verArr[count($verArr)-1] = 0;
						$verArr[count($verArr)-2] = $verArr[count($verArr)-2]+1;
					}elseif($verArr[count($verArr)-3]<=9){
						$verArr[count($verArr)-1] = 0;
						$verArr[count($verArr)-2] = 0;
						$verArr[count($verArr)-3] = $verArr[count($verArr)-3]+1;
					}
					$version = implode(".", $verArr);
				}
			}
			
			
			
			$host = pwaforwp_home_url();
			$packageId = $defaults['pta_package_id']; 
			
			$data = array(
			    "packageId"=> $packageId,
			    "host"=> $host,
			    "name"=> esc_attr($defaults['app_blog_name']),
			    "launcherName"=> esc_attr($defaults['app_blog_short_name']),
			    "themeColor"=> sanitize_hex_color($defaults['theme_color']),
			    "navigationColor"=> sanitize_hex_color($defaults['background_color']),
			    "backgroundColor"=> sanitize_hex_color($defaults['background_color']),
			    "startUrl"=> "/",
				"webManifestUrl"=>esc_url( pwaforwp_manifest_json_url() ),
			    "iconUrl"=> esc_url(pwaforwp_https($defaults['icon'])),
			    "maskableIconUrl"=> esc_url(pwaforwp_https($defaults['splash_icon'])),
			    "appVersion"=> $version,
			    "appVersionCode"=> $appVersionCode,
			    "useBrowserOnChromeOS"=> true,
			    "splashScreenFadeOutDuration"=> 300,
			    "enableNotifications"=> true,
				"display"=> esc_attr($defaults['display']),
			    "shortcuts"=> array(array(
								"name"=> esc_attr($defaults['app_blog_name']),
								"short_name"=> esc_attr($defaults['app_blog_short_name']),
								"url"=> "/?shortcut",
								"icons"=> array( /*array(
										"sizes"=> "128x128",
										"src"=> "/favicon.png",
									) */)
							)),
				"signingMode"=> $signingMode,
			    "signing"=> array(
					"file"=> $keystorefile,
					"alias"=> "my-key-alias",
			        "fullName"=> "PWA for wp",
			        "organization"=> "MAGAZINE3",
			        "organizationalUnit"=> "Development",
			        "countryCode"=> "IN",
					"keyPassword"=> $keyPassword,
					"storePassword"=> $keystorePassword
			    ),
				"fallbackType"=> "customtabs",
			);

			$headers_data = array('request_url'=>esc_url(get_home_url()), 'request_license'=>sanitize_text_field($this->settings['ptafp_addon_license_key']));
			set_time_limit(500);
			$response = $this->ptap_authentication->sender_request("generateAppProject", $data, $headers_data, true);
			if($response){
				if( $response['status'] == 200){
					$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
					 if(strpos($host, 'www')!==false){
						$host = str_replace("www.", "", $host);
					} 
					
					$upload_path = wp_upload_dir();
					$file_path = $upload_path['basedir']."/pwa_to_app/";
					wp_mkdir_p($file_path);
					$file_name = $host."-project.zip";
					$file_name_path = $file_path.'/'.$file_name;
					if(file_exists($file_name_path)){ @unlink($file_name_path); }
					$fp = file_put_contents($file_name_path, $response['body']);

					$fileUrl = $upload_path['baseurl']."/pwa_to_app/".$file_name;
					echo json_encode(array("status"=>200, "message"=> esc_html__("App project generated",'pwa-to-apk-plugin')." <a href='".$fileUrl."'>".esc_html__("Download", 'pwa-to-apk-plugin')."</a>"));
				}else{
					echo json_encode(array("status"=>300, "message"=> esc_html__("App project not generated try later.", 'pwa-to-apk-plugin')." <br/>ErrorDetails: ".$response['body']));
				}
			}else{
				echo json_encode(array("status"=>400, "message"=> "Some error occurred in response, try again later."));
			}
			die;
		}

		function init(){
			if(!defined('PWAFORWP_PLUGIN_VERSION')){
				add_action( 'admin_notices', array( $this, 'pwa_parent_plugin_notice' ) );
			}else{
				//Add default Settings
				add_filter("pwaforwp_final_settings_vals", array($this, 'pwa_to_apk_default_settings'), 10, 1);

				add_settings_section('pwaforwp_pwa_to_apk_plugin_setting_section', esc_html__('','pwa-for-wp'), '__return_false', 'pwaforwp_pwa_to_apk_plugin_setting_section');
				$pluginStatus = $this->plugin_status_check();
				if($pluginStatus){
					add_settings_field(
						'pwaforwp_pwa_apk_packageId_setting', // ID
						esc_html__('Package Id', 'pwa-for-wp'),	// Title
						array($this, 'pwa_to_apk_packageId_setting_callback'),	// CB
						'pwaforwp_pwa_to_apk_plugin_setting_section',// Page slug
						'pwaforwp_pwa_to_apk_plugin_setting_section'// Settings Section ID
					);

					add_settings_field(
						'pwaforwp_pwa_apk_accepted_setting', // ID
						esc_html__('Generate APK type', 'pwa-for-wp'),	// Title
						array($this, 'pwa_to_apk_accepted_setting_callback'),	// CB
						'pwaforwp_pwa_to_apk_plugin_setting_section',// Page slug
						'pwaforwp_pwa_to_apk_plugin_setting_section'// Settings Section ID
					);
				}
				add_settings_field(
					'pwaforwp_pwa_apk_set_setting', // ID
					esc_html__('Generate PWA APP', 'pwa-for-wp'),	// Title
					array($this, 'pwa_to_apk_plugin_setting_callback'),	// CB
					'pwaforwp_pwa_to_apk_plugin_setting_section',// Page slug
					'pwaforwp_pwa_to_apk_plugin_setting_section'// Settings Section ID
				);
				add_settings_field(
					'pwaforwp_pwa_apk_project_file_setting', // ID
					esc_html__(' ', 'pwa-for-wp'),	// Title
					array($this, 'pwa_to_apk_plugin_project_callback'),	// CB
					'pwaforwp_pwa_to_apk_plugin_setting_section',// Page slug
					'pwaforwp_pwa_to_apk_plugin_setting_section'// Settings Section ID
				);
			}
			
		}

		function pwa_to_apk_default_settings($pwaforwp_settings){
			if(!isset($settings['pta_package_id'])){
				$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
				if(strpos($host, 'www')!==false){
					$host = str_replace("www.", "", $host);
				}
				$packageId = preg_replace("/[^a-z0-9]/", "", $host); 
				
				
				$pwaforwp_settings['pta_package_id'] = "com.".$packageId.".myapp";
			}
			return $pwaforwp_settings;
		}

		public function pwa_to_apk_accepted_setting_callback(){
			$settings = $this->settings;
			$newapktype = 'checked'; $mineapktype = '';
			if(isset($settings['apk_type']) && $settings['apk_type']=='mine'){
				$newapktype = ''; $mineapktype = 'checked';
			}
			echo "<label style='display:inline-block'>".esc_html__('Create first time', 'pwa-for-wp')." <input type='radio' name='pwaforwp_settings[apk_type]' class='generate_apk_type' value='new' ".$newapktype."></label>
			<label style='display:inline-block'>".esc_html__('Create new version of APK', 'pwa-for-wp')." <input type='radio' name='pwaforwp_settings[apk_type]' class='generate_apk_type' value='mine' ".$mineapktype."></label>";
			echo "<p>".esc_html__('If you have already submitted APK on Google Play store, then select old and fill the below details. You can find these details in your old zip of APK', 'pwa-for-wp')."</p>";
			$showOldOption = false;
			if(isset($settings['apk_type']) && $settings['apk_type']=='mine'){
				$showOldOption = true;
			}
			$showUploadKeyStore = '';$uploadedStat = '';
			if(file_exists(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore")){
				$showUploadKeyStore = 'hide';$uploadedStat ="<span class='dashicons dashicons-yes'  id='keystore-file'></span><em class='pwaupload_new_keystore'> Upload new file</em>";
			}

			echo "
			<style>.signedInfo-wrapper .field{padding-top:5px;}.signedInfo-wrapper .field label{display:inline-block;width:30%;}.pwaupload_new_keystore{cursor:pointer;}.signedInfo-wrapper .hide{display:none;}</style>
			<div class='signedInfo-wrapper' ".( $showOldOption? '': 'style="display:none"' ).">
				<div class='field'><label>".esc_html__('Last App version', 'pwa-for-wp')." </label> <input type='text' name='pwaforwp_settings[apk_app_version]' value='".@$settings['apk_app_version']."'></div> 
				<div class='field'><label>".esc_html__('Last App version code', 'pwa-for-wp')." </label><input type='number' name='pwaforwp_settings[apk_app_code]' value='".@$settings['apk_app_code']."'> </div>
				<hr/>
				<div class='field'><label>".esc_html__('Key file', 'pwa-for-wp')."</label> 
				".$uploadedStat."
				<span class='keystore ".$showUploadKeyStore."'><input type='file' accept='.keystore'> <input type='button' value='Upload' class='pwa_uploadkey_store'><span class='response_msg'></span></span>
				</div>

				<div class='field'><label>".esc_html__('Key password', 'pwa-for-wp')." </label> <input type='text' name='pwaforwp_settings[apk_key_password]' value='".@$settings['apk_key_password']."'> </div>
				<div class='field'><label>".esc_html__('Key store password', 'pwa-for-wp')." </label> <input type='text' name='pwaforwp_settings[apk_key_store_password]' value='".@$settings['apk_key_store_password']."'></div>
			</div>";
		}

		function pwa_to_apk_packageId_setting_callback(){
			$settings = $this->settings;
			if(!isset($settings['pta_package_id'])){
				$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
				if(strpos($host, 'www')!==false){
					$host = str_replace("www.", "", $host);
				}
				$packageId = preg_replace("/[^a-z0-9]/", "", $host); 
				
				$packageId = "com.".$packageId.".myapp";
			}else{
				$packageId = $settings['pta_package_id'];
			}
			?>
			<input type="text" name="pwaforwp_settings[pta_package_id]" id="pwaforwp_settings[pta_package_id]" class=""  value="<?php echo esc_attr($packageId); ?>">
			<?php
		}

		function pwa_to_apk_plugin_setting_callback(){
			$settings = $this->settings;
			$pluginStatus = $this->plugin_status_check();
			if($pluginStatus){
				$pwa_pwatapk = get_option("pwa_pwatapk", true);
				$olderDownload = '';
				$nonce = wp_create_nonce("pwa_pwatapk_nonce_assetlink");
				if(isset($pwa_pwatapk['old_files']) && $pwa_pwatapk['old_files']){
					foreach (array_reverse($pwa_pwatapk['old_files']) as $key => $value) {
						$upload_path = wp_upload_dir();
						$file_name = isset($value['fname'])? $value['fname'] : '';
						if(!isset($value['fname'])){
							$nameArr = explode('/', $value['name']);
							$file_name = end($nameArr);
						}
						$fileUrl = $upload_path['baseurl']."/pwa_to_app/".$file_name;
						$olderDownload .= '<div class="apkzip-row" style="padding-top:8px"> 
									<span style="width:48%;display:inline-block;">'.$value['version'].'</span>
									<span style="width:24%;display:inline-block;">
										<a target="_blank" href="'.$fileUrl.'">DOWNLOAD</a>
									</span>
									<span style="width:24%;display:none;">
										<em data-nonce="'.$nonce.'" data-version="'.$value['version'].'" class="delete_old_pwa" href="#" >Delete</em>
									</span>
								</div>';//&nbsp;&nbsp; <span class="putassetsLive" data-nonce="'.$nonce.'" class="update-message" data-version="'.$value['version'].'" style="cursor:pointer"> Make asset live <p style="display:none"></p></span>
					}
				}
				?>
				<div class="download_wrapper">
					<button type="button" id="pwaforwp_pwa_to_apk_plugin_sets" class="download_apk" data-nonce="<?php echo wp_create_nonce("pwa_pwatapk_nonce_gen"); ?>">Download APK</button>
					<span class="update_message"><p style="display:inline"></p></span>
				</div>
				<?php
				if($olderDownload){
					echo "<br/>
						  <br/><hr/><strong>Previous generated APK</strong>";
					echo "<div style='font-weight:600;padding-bottom:5px; padding-top:5px'><span style='width:48%;display:inline-block;'>Version</span><span style='width:48%;display:inline-block;'>Action</span></div>";
					echo "<div style='height:80px;max-height:80px;overflow-y:scroll'>".$olderDownload."</div>";
				}
			}else{
				?>
				<input type="button"  id="pwaforwp_pwa_to_apk_plugin_setting_section" class="" value="Download APK" disabled>
				<p>Please Activate the plugin <a href="<?php echo admin_url('admin.php?page=pwaforwp&tab=premium_features'); ?>">Activate now</a></p>
				<?php
			}
			?>              
		        
			<?php
		}
		
		function pwa_to_apk_plugin_project_callback(){
			$host = parse_url(pwaforwp_home_url(), PHP_URL_HOST);
				 if(strpos($host, 'www')!==false){
					$host = str_replace("www.", "", $host);
				} 
				
				$upload_path = wp_upload_dir();
				$file_path = $upload_path['basedir']."/pwa_to_app/";
				$file_path_uri = $upload_path['baseurl']."/pwa_to_app/";
				wp_mkdir_p($file_path);
				$file_name = $host."-project.zip";
				$file_name_path = $file_path.'/'.$file_name;
				$file_path_uri = $file_path_uri.'/'.$file_name;
				if(file_exists($file_name_path)){ 
					$previousFile = "<a href='$file_path_uri'>".esc_html__('Click here to download', 'pwa-to-apk-plugin')."</a>";
				}
				?>
				<div class="download_wrapper">
					<button type="button" id="pwaforwp_pwa_to_apk_plugin_project" class="download_apk_project" data-nonce="<?php echo wp_create_nonce("pwa_pwatapk_nonce_gen"); ?>"><?php echo esc_html__("Download APK project (For development)", 'pwa-to-apk-plugin')?></button>
					<span class="update_message"><p style="display:inline"><?php echo $previousFile; ?></p></span>
				</div>
				<?php
				
		}

		function plugin_status_check(){
			$license_key = $license_status = $license_status_msg = '';
			if( isset($this->settings['ptafp_addon_license_key']) ){
                  $license_key =   $this->settings['ptafp_addon_license_key'];
            }
            
            if(isset($this->settings['ptafp_addon_license_key_status'])){
              $license_status =   $this->settings['ptafp_addon_license_key_status'];
            }
            
            if(isset($this->settings['ptafp_addon_license_key_message'])){
              $license_status_msg =   $this->settings['ptafp_addon_license_key_message'];
            }

            $pwa_pwatapk = get_option("pwa_pwatapk", true);
            if( $license_status == 'active' && !empty($license_key) && isset($pwa_pwatapk["user_authentication"]) && $pwa_pwatapk["user_authentication"]==1 ){
            	return true;
            }
            return false;
		}

		

		function script_enqueue($hook){
			if($hook== 'toplevel_page_pwaforwp'){ 
	            wp_enqueue_script(
	                'pwaforwp-ptapwp-switches',
	                PWAFORWP_PTAFP_PLUGIN_URL . '/assets/admin-pwa-tp-apk-plugin.js',
	                array( 'jquery','pwaforwp-main-js' ),
	                PWAFORWP_PTAFP_VERSION,
	                true
	            );
	        }
		}


		function pwa_parent_plugin_notice(){
			$class = 'notice notice-error';
			$message = esc_html__( 'Parent plugin PWA for WP & AMP not activated.', 'pwa-to-apk-plugin' );

			printf( '<div class="%1$s"><p>%2$s <a href="%3$s" class="button button-primary" target="__blank">Click here</a></p></div>', esc_attr( $class ),  $message , esc_url('https://wordpress.org/plugins/pwa-for-wp/') ); 
		}

		function plugin_action_links( $links ) {
			$links[] = '<a href="' . esc_url( admin_url( 'admin.php?page=pwaforwp&tab=features' ) ) . '">' . esc_html__( 'Settings', 'pwa-for-wp' ) . '</a>';	
		        $links[] = '<a target="_blank" href="http://pwa-for-wp.com/docs/">' . esc_html__( 'Documentation', 'pwa-to-apk-plugin' ) . '</a>';
		  	return $links;
		        
		}
		
		function pwa_to_apk_assets_deeeplink($version=''){
			//$version = $_GET['version'];
			//echo $version;die;
			if(empty($version)){ return ;}
			$status = 300; $message = "files not found";

			$wprootpath  = str_replace("//","/",str_replace("\\","/",realpath(ABSPATH))."/"); 
			$wprootpath  = apply_filters("pwaforwp_file_creation_path", $wprootpath);

			$destination = $wprootpath."/.well-known/assetlinks.json";
			if(!file_exists($destination)){
				$pwa_pwatapk = get_option("pwa_pwatapk", true);
				if(isset($pwa_pwatapk['old_files']) && $pwa_pwatapk['old_files']){
					$filedir = '';
					$upload_path = wp_upload_dir();
					foreach (array_reverse($pwa_pwatapk['old_files']) as $key => $value) {
						if($value['version']==$version){
							$file_name = isset($value['fname'])? $value['fname'] : '';
							if(!isset($value['fname'])){
								$nameArr = explode('/', $value['name']);
								$file_name = end($nameArr);
							}
							$filedir = $upload_path['basedir']."/pwa_to_app/".$file_name;
							break;
						}
					}
					wp_mkdir_p($upload_path['basedir']."/pwa_to_app/tmp/");
					$extracted = unzip_file($filedir, $upload_path['basedir']."/pwa_to_app/tmp/");
					if($extracted){
						$source = $upload_path['basedir']."/pwa_to_app/tmp/assetlinks.json";
						wp_mkdir_p($wprootpath."/.well-known");
						if(file_exists($source)){
							$copied = copy($source, $destination);
							if($copied){
								$status = 200; $message = "file moved successful";
							}else{
								$status = 200; $message = "file not moved";
							}
						}else{
							$status = 300; $message = "file not found";
						}
					}else{
						$status = 300; $message = "zip file not extracted";
					}
					
				}
			}
			if(!file_exists(PWAFORWP_PTAFP_PLUGIN_DIR.'/keystore/signing.keystore')){
				$upload_path = wp_upload_dir();
				$source = $upload_path['basedir']."/pwa_to_app/tmp/signing.json";
				if(file_exists($source)){
					$destination = PWAFORWP_PTAFP_PLUGIN_DIR.'/keystore/signing.keystore';
					if(!is_dir(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/")){
			        	wp_mkdir_p(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/");
			        }
			        copy($source, $destination);
			    }
			}
			$settings = $this->settings;
			if(empty($settings['apk_key_password']) || empty($settings['apk_key_store_password'])){
				$upload_path = wp_upload_dir();
				$sourceFile = $upload_path['basedir']."/pwa_to_app/tmp/signing-key-info.txt";
				$fh = fopen($sourceFile,'r');
				while ($line = fgets($fh)) {
					if(strpos($line, ":")!==false){
						$lineData = explode(": ", $line);
						if($lineData[0]=='Key store password' && isset($lineData[1])){
							$settings['apk_key_password'] = trim($lineData[1]);
						}
						if($lineData[0]=='Key password' && isset($lineData[1])){
							$settings['apk_key_store_password'] = trim($lineData[1]);
						}
					}
				}
				update_option( 'pwaforwp_settings', $settings ); 
				fclose($fh);
			}
			
			



			
			error_log($message);
			//echo json_encode(array("status"=>$status, "message"=>$message));die;
		}

		function pwaforwp_apk_keystore_upload(){
			if ( ! isset( $_GET['pwaforwp_security_nonce'] ) ){
	           echo json_encode(array("status"=>500, "message"=> "nonce not found"));
	            die;
	        }
	        if ( !wp_verify_nonce( $_GET['pwaforwp_security_nonce'], 'pwaforwp_ajax_check_nonce' ) ){
	           echo json_encode(array("status"=>500, "message"=> "nonce not matched"));
	           die;
	        }
	        $keystore_file = $_FILES['keystore_file'];
	        if($keystore_file['error']>0){
	        	echo json_encode(array("status"=>400, "message"=> "file not found"));
	           die;
	        }
	        if(!is_dir(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/")){
	        	wp_mkdir_p(PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/");
	        }
	        $movedFile = move_uploaded_file ( $keystore_file['tmp_name'] , PWAFORWP_PTAFP_PLUGIN_DIR."/keystore/signing.keystore"); 
	        if($movedFile){
	        	echo json_encode(array("status"=>200, "message"=> "File upload successful"));
	           die;
	        }else{
	        	echo json_encode(array("status"=>201, "message"=> "File not uploaded try again"));
	           die;
	        }
		}

		function pwa_to_apk_assets_removeold(){
			$nonce = $_POST['ref'];
			$version = $_POST['oldversion'];
			if(!wp_verify_nonce($nonce, 'pwa_pwatapk_nonce_assetlink')){
				echo json_encode(array("status"=>400, "message"=>"Request not verified"));die;
			}	
			$pwa_pwatapk = get_option("pwa_pwatapk", true);
			if(isset($pwa_pwatapk['old_files']) && $pwa_pwatapk['old_files']){
				$filedir = '';
				$upload_path = wp_upload_dir();
				foreach ($pwa_pwatapk['old_files'] as $key => $value) {
					if($value['version']==$version){
						$file_name = isset($value['fname'])? $value['fname'] : '';
						if(!isset($value['fname'])){
							$nameArr = explode('/', $value['name']);
							$file_name = end($nameArr);
						}
						$fileUrl = $upload_path['basedir']."/pwa_to_app/".$file_name;
						if(file_exists($fileUrl)){ @unlink($fileUrl); }
						unset($pwa_pwatapk['old_files'][$key]);
						break;
					}
				}
				update_option("pwa_pwatapk", $pwa_pwatapk);
			}
			echo json_encode(array("status"=>200, "message"=>"Version removed successfully"));die;
		}
		
		function generate_random_string($length = 5) {
			$characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
			$charactersLength = strlen($characters);
			$randomString = '';
			for ($i = 0; $i < $length; $i++) {
				$randomString .= $characters[rand(0, $charactersLength - 1)];
			}
			return $randomString;
		}

		
	}
	$pwaforwp_ptafp = new pwaforwp_ptafp();
}